#!/bin/bash
##############################################################################
# (C) Copyright 2017 Hewlett Packard Enterprise Development LP.
#     confidential commercial computer software.
#
#
##############################################################################
# db2tkit_utils.sh : HP Serviceguard toolkit for DB2 - common utility script
#
##############################################################################
# Variables
export Instance_DB_Array=()

##############################
# Source SG utility functions.
##############################
export HOST=`/bin/hostname`
SG_UTILS=$SGCONF/scripts/mscripts/utils.sh

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if [[ $? -ne 0 ]]
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
# Get the environment for this package through utility function
# sg_source_pkg_env().
###################################################################

export PATH=$PATH:$SGSBIN

if [[ ! -f $2 ]] ; then
        sg_source_pkg_env $1
else
        sg_source_pkg_env $*
fi

export INSTANCE_HOME=`eval echo "~${DB2_INSTANCE_NAME}"`

###############################################################################
# Function: check_return
# Check return value of functions and exit. It accepts two parameters.
# Parameters:
# 1. Exit value of the previously executed command # 2. The place of execution
#        1 - db2_run_cmds
#        2 - db2_halt_cmds
#        3 - db2 Monitor script
#        4 - Improper parameters to toolkit script
###############################################################################

function check_return
{
    if [[ $1 -ne 0 ]]; then
      case $2 in
        1)
             log_message "ERROR: Function db2_run_cmds: Failed to start the DB2 instance"
             db2_halt_cmds
             exit 1
        ;;

        2)
             log_message "ERROR: Function db2_halt_cmds: Failed to stop the DB2 instance"
             exit_code=1
        ;;

        3)   log_message "ERROR: Failed to start the DB2 monitor script"
             exit 1
        ;;

        4)   log_message "ERROR: Improper parameters in DB2 package configuration file."
             exit 1
        ;;

        *)
             log_message "ERROR: Failed, unknown error."
             exit 1
             ;;
      esac
    fi
}

###############################################################################
# Function: log_message
#
# This function log_messagees any message with date, time and node name affixed
# to it. It accepts just one parmeter.
# Parameter:
# 1. Message to be logged
###############################################################################

function log_message
{
    if [[ $# -ge 1 ]]; then
        echo "$(date '+%b %e %T') - Node \"${HOST}\" $* "
    fi
}

###############################################################################
# Function    : kill_hung_processes
# Description : If the database is hung, spawn a background process that will
#               issue a SIGKILL to each of the monitored processes This allows
#                us to guarantee a time limit for clean halt attempts.
# Arguments   : None
# Returns     : None
###############################################################################

function kill_hung_processes
{

#   Wait for the TIME_OUT period before sending a kill signal.  The TIME_OUT
#   value should be less than the HP Serviceguard package time-out.
    flag=0
    sleep ${DB2_TIME_OUT}

    for i in ${DB2_MONITOR_PROCESSES[@]}
    do
      for j in ${DB2_PARTITION_NUMBER[@]}
      do
        id=`ps -fu "${DB2_INSTANCE_NAME}" | grep "${j}[ ]*$" | awk '/'${i}'/ { print $2 }'`
        if [[ ${id} != "" ]]
        then
            kill -9 ${id}
            check_status=$?
            if [[ $check_status -ne 0 ]]
            then
                log_message "kill_hung_processes function could not kill ${i} process."
                flag=1
            else
                log_message "kill_hung_processes function did find process ${i} running."\
                "sending SIGKILL."
            fi
        else
            log_message "kill_hung_processes function did not find any process running."
        fi
      done
    done

    return $flag

}

#######################################################################
# Function    : verify_unsupported_online_modification
# Description : This function verifies if there is an attempt by user
#               to modify the attribute values that are currently not
#               supported by toolkit to be online reconfigurable.
# Arguments   : None
# Returns     : 1 - if unsupported attributes are modified.
#######################################################################
function verify_unsupported_online_modification
{
  typeset -i i=0
  typeset -i flag=0

 # Verify if any PARITION attribute value is added,modified or deleted.
 cur_pn=`echo ${DB2_PARTITION_NUMBER[*]} | sed 's/ //g'`
 prev_pn=`echo ${PREV_DB2_PARTITION_NUMBER[*]} | sed 's/ //g'`

 if [[ "$cur_pn" != " " || "$prev_pn" != " " ]]; then
    if [[ "$cur_pn" != "$prev_pn" ]]; then
       flag=1
       sg_log 0 "ERROR: Online modification of PARTITION_NUMBER attribute value is not supported"
    fi
 fi

 if [[ ${DB2_INSTANCE_NAME} != ${PREV_DB2_INSTANCE_NAME} ]]; then
    sg_log 0 "ERROR: Online modification of DB2_INSTANCE_NAME attribute value is not supported"
    flag=1
 fi

 if [[ ${DB2_INSTANCE_USER} != ${PREV_DB2_INSTANCE_USER} ]]; then
    sg_log 0 "ERROR: Online modification of DB2_INSTANCE_USER attribute value is not supported"
    flag=1
 fi

 if [[ $flag -eq 1 ]]; then
    return 1
 fi

}


#######################################################################
# Function    : get_service_name
# Description : This function gets the service name for restarting during
#               online reconfiguration.
# Arguments   : None
# Returns     : 0 - if service is found.
#######################################################################
function get_service_name
{
   if [[ ${#SG_SERVICE_NAME[@]} -ne 0 ]]; then
       for (( i=0; i<${#SG_SERVICE_NAME[@]}; i++ ))
       do
           FOUND=`echo "${SG_SERVICE_CMD[$i]}" | grep -w 'db2_monitor'` > /dev/null 2>&1
           if [[ "$FOUND" != "" ]]
           then
               service_name=${SG_SERVICE_NAME[$i]}
               service_cmd=${SG_SERVICE_CMD[$i]}
               sg_log 5 "Found monitoring service ${service_name}"
               return
           fi
       done
   fi
}

###############################################################################
# Function    : validate_reconfigure_restart
# Description : This function validates the DB2 package attributes for online
#               modification.
# Arguments   : None
# Returns     : 0 - if no change in attribute value.
#               1 - if there is a change in the attribute value.
###############################################################################
function validate_reconfigure_restart
{

    if [[ ${DB2_MONITOR_INTERVAL} = ${PREV_DB2_MONITOR_INTERVAL} ]] &&
       [[ ${DB2_TIME_OUT} = ${PREV_DB2_TIME_OUT} ]] &&
       [[ ${SG_SCRIPT_LOG_FILE} = ${PREV_SG_SCRIPT_LOG_FILE} ]] &&
       [[ ${SG_EMAIL_ID[*]} =  ${PREV_SG_EMAIL_ID[*]} ]]
    then
        return 0
    fi
    return 1
}


###############################################################################
# Function    : validate_shared_storage
# Description : This function validates if instance home is on shared disk or 
#                not
# Arguments   : name of the directory 
# Returns     : 0 - if storage is not shared disk
#               1 - if storage is shared disk
###############################################################################

function validate_shared_storage
{
#Check if the instance home is shared.
  inst_on_shared_disk=0
  # Search through FS directory to see if it has instance home shared
  if [[ -n "${SG_FS_DIRECTORY[@]}" ]] ; then
     for (( i=0; i<${#SG_FS_DIRECTORY[@]}; i++)) ; do
        if echo ${1} | grep -q -w "^${SG_FS_DIRECTORY[$i]}" 2>/dev/null ; then
           inst_on_shared_disk=1
           break
        fi
     done
  fi
  return $inst_on_shared_disk
}

###############################################################################
# Function    : verify_if_db_exist
# Description : This function verifies if there is any db(s) associated for a 
#               for a given instance.
# Arguments   : N/A
# Returns     : 0 - if atleast one db is available.
#               1 - if unable to find any db.
###############################################################################

function verify_if_db_exist
{
    # login for a given instance & get the associated DB's
    su - ${DB2_INSTANCE_USER} -c "/bin/sh" << DB2_DB_VERIFY >/dev/null 2>&1
    $INSTANCE_HOME/sqllib/bin/db2 list database directory \
                          | grep -B6 -i indirect | grep "Database name"
    exit \$?
DB2_DB_VERIFY
    RC=$?
    if [[ $RC -ne 0 ]]; then
        return 1
    else
        return 0
    fi
}


###############################################################################
# Function    : get_instance_dbs
# Description : This function verifies & list all the db's associated with an
#               instance
# Arguments   : N/A
# Returns     : 0 - if atleast one db is available.
#               1 - if unable to find any db.
###############################################################################

function get_instance_dbs
{
    # Verify if a DB exist for a given instance name
    verify_if_db_exist
    if [[ $? -ne 0 ]]; then
        return 1
    fi

    # Define a file variable to write the DBS name
    export tmpfile="/var/tmp/${SG_PACKAGE_NAME}_db2_db_names.txt"
    if [[ -f $tmpfile ]]; then
        rm -f $tmpfile
    fi
    touch $tmpfile
    chmod 777 $tmpfile
	
    # login for a given instance & get the associated DB's
    su - ${DB2_INSTANCE_USER} -c "/bin/sh" << DB2_GET_DB >/dev/null 2>&1
    DBS=\`$INSTANCE_HOME/sqllib/bin/db2 list database directory show detail \
       | grep -B6 -i indirect | grep "Database name"\`
    ret_val=\$?
    if [[ \$ret_val -eq 0 ]]; then
        echo "\$DBS" | awk -F"=" '{print \$2}'> $tmpfile
    fi
    exit \$ret_val
DB2_GET_DB
    RC=$?
    if [[ $RC -ne 0 ]]; then
        return 1
    else
        IFS='
        '
        Instance_DB_Array=( $( < ${tmpfile} ) )
        rm -f $tmpfile

        if [[ ${#Instance_DB_Array[@]} -eq 0 ]]; then
            sg_log 5 "ERROR: file ${tmpfile} is empty"
            return 1
        fi
    fi
    return 0
}

###############################################################################
# Function    : verify_db_path_on_shared_storage
# Description : This function verifies if DB is on shared disk.
# Arguments   : N/A
# Returns     : 0 - if it is not on shared storage
#               1 - if atleast one DB is on shared storage
###############################################################################
function verify_db_path_on_shared_storage
{
  flag=0 
  ret=0
  ret_val=0
# Define a file variable to write the DBS name
    export tmpfile="/var/tmp/${SG_PACKAGE_NAME}_db2_db_paths.txt"
    if [[ -f $tmpfile ]]; then
        rm -f $tmpfile
    fi
    touch $tmpfile
    chmod 777 $tmpfile

    su - ${DB2_INSTANCE_USER} -c "/bin/sh" << DB2_GET_DB
    DBS=\`${INSTANCE_HOME}/sqllib/bin/db2 list database directory | grep "database directory" | grep -v empty\`
    if [[ -n "\$DBS" ]]; then
       echo "\$DBS" | awk -F"=" '{print \$2}' > $tmpfile
    else
       echo "INFO: Database path does not exists. Skipping the validation for DB existence."
       ret_val=1
    fi
    exit \$ret_val
DB2_GET_DB
    if [[ $? -eq 0 ]]; then
        IFS='
        '
        Instance_DBPATH_Array=( $( < ${tmpfile} ) )
        rm -f $tmpfile

        for i in ${Instance_DBPATH_Array[@]}
        do
             validate_shared_storage $i
             ret=$?
             if [[ $ret -eq 0  && ${DB2_HANG_DETECT} = "yes" && "${DB2_HANG_DETECT_LEVEL}" -gt 1 ]]; then
                verify_if_db_exist
                if [[ $? -ne 0 ]]; then
                   sg_log 0 "ERROR: Unable to find any DB configured for the Instance ${DB2_INSTANCE_NAME}"\
                   "since attribute HANG_DETECT is set to YES, atleast one DB should be available"
                   flag=1 
                fi
             elif [[ $ret -ne 0 && ${DB2_HANG_DETECT} = "yes" && "${DB2_HANG_DETECT_LEVEL}" -gt 1 ]]; then
                  sg_log 0 "INFO: Skipping validation for the database existence as database is on shared disk"
                  continue
             fi

         done
    fi
    if [[ $flag -eq 1 ]]; then
       return 1
    fi
    return 0
}
