#!/bin/bash
#############################################################################
# (C) Copyright 2017 Hewlett Packard Enterprise Development LP.
#     confidential commercial computer software.
#
##############################################################################
# hadb2.sh : Serviceguard toolkit for DB2 startup/shutdown script
#
################################################################################
# Source cluster environment file
if [[ -f /etc/cmcluster.conf ]]; then
        . /etc/cmcluster.conf
fi

# Source SG_UTILS & DB2 Toolkit global variables/functions
if [[ -f $SGROOT/db2toolkit/db2tkit_utils.sh ]]; then
        . $SGROOT/db2toolkit/db2tkit_utils.sh
fi


###############################################################################
# Function   : vaidate_db2
# Description: validates the DB2 package parameters  
# Returns    : 1 - If any error and validation fails.
################################################################################
function validate_db2
{


 if [[ -z $SG_PRIV_CLUSTER_VERIFICATION  || $SG_COMMAND = "cmapplyconf" ]]; then
    # Check if package is applied
    $SGSBIN/cmviewcl -f line -s config -p $SG_PACKAGE > /dev/null 2>&1
    if [[ $? -eq 0 ]]; then
       # Check if package is running and check if this modules is not newly getting
       # added during online configuration, then only call function verify_unsupported_online_modification
       pkg_stat=`$SGSBIN/cmviewcl -f line | grep "$SG_PACKAGE|status" | cut -d "=" -f 2 2>&1`
       if [[ "$pkg_stat" = "up" && ${PREV_SG_MODULE_NAME[@]} = *"tkit/db2"* ]]; then
          verify_unsupported_online_modification
          if [[ $? -eq 1 ]]; then
             return 1
          fi
       fi
    fi
 fi 

  typeset -i err_flag=0
  # Confirm that the DB2 user exists
  if ! id $DB2_INSTANCE_USER > /dev/null 2>&1 ; then
     sg_log 0 "ERROR: DB2 user $DB2_INSTANCE_USER does not exists"
     sg_log 0 " - Add user $DB2_INSTANCE_USER on the node"
     err_flag=1
  fi


  #Check if the instance home is shared.
  validate_shared_storage ${INSTANCE_HOME}
  if [[ $? -eq 0 ]];then
    #Check if instance binaries are present in the instance home directory derived.
    if [[ ! -d ${INSTANCE_HOME}/sqllib ]]; then
       sg_log 0 "ERROR: DB2 instance binaries does not exists in the directory ${INSTANCE_HOME}."
       err_flag=1
    fi 

    # Check whether the db2gcf binary is present. 
    # Note: db2gcf is important binary in db2 used for starting, stopping and monitoring
    # db2 instances.
    if [[ ! -s ${INSTANCE_HOME}/sqllib/bin/db2gcf ]]; then
       sg_log 0 "ERROR:$INSTANCE_HOME/sqllib/bin/db2gcf does not exists."
       err_flag=1
    fi

    #Check for the existence of db2nodes.cfg configuration file.
    if [[ ! -s ${INSTANCE_HOME}/sqllib/db2nodes.cfg ]]; then
       sg_log 0 "ERROR:$INSTANCE_HOME/sqllib/db2nodes.cfg file does not exists or empty."
       err_flag=1
    fi

  else
    sg_log 0 "INFO: Skipping the validation for DB2 instance binary and configuration"\
             "files as instance home is located on the shared storage" 
  fi 

  #validate if the port required for DB2 instance is avaiable in the /etc/services file.
  #minimum  5 ports should be reserved for any DB2 instance.
  port_presence=`cat /etc/services | awk '{print $1}' | awk -F "_" '{print $2}' | grep -w ${DB2_INSTANCE_NAME} | wc -l`
  if [[ $port_presence -lt 5 ]]; then
     sg_log 0 "ERROR: /etc/services does not contain the minimum ports required for"\
              "instance ${DB2_INSTANCE_NAME}"
     sg_log 0 "- Please specify the required ports for the ${DB2_INSTANCE_NAME}"\
              "instance in /etc/services file in all the packaged nodes."
     err_flag=1 
  fi

  ## Verify the hang detection script required attributes with the following conditions-
  #  1. If the DB2_HANG_DETECT attribute is set to YES. Then verify if script exists
  #  2. Detect the hang detect level value. Call hang_detect only if LEVEL>1.
  #  3. Verify defined value for the DB2_HANG_DETECT_EMAIL attribute
  
  # 1. If the DB2_HANG_DETECT attribute is set to YES. Then verify if script exists
  if [[ "${DB2_HANG_DETECT}" == "YES" ]]; then

     # Error out if db2ls binary is not present.
     if [[ ! -s /usr/local/bin/db2ls ]]; then

        sg_log 0 "ERROR: /usr/local/bin/db2ls command does not exists."
        sg_log 0 "- Please create the symbolic link /usr/local/bin/db2ls to the db2ls command"\
                 "available inside the DB2 home directory."
        err_flag=1
		
     # Verify the symbolic link target details & generate the hang detect script path.
     else
	 
        # Get the symbolic link target directory
        DB2LSS_DIR=`readlink /usr/local/bin/db2ls`
        if [[ $? -ne 0 ]]; then
            sg_log 0 "ERROR: Please verify if the symbolic link has been created"\
	   	                "correctly for the installed db2ls binary."
            err_flag=1
        else
            # Get the DB2 default hang detect script
            DB2_HANG_DET_SCRIPT_PATH=`echo ${DB2LSS_DIR} | sed 's/install\/db2ls/samples\/pd\/db2_hang_detect/'`
            if [[ ! -s ${DB2_HANG_DET_SCRIPT_PATH} ]]; then
                sg_log 0 "ERROR: Unable to get the default DB2 db2_hang_detect"\
                                  "script using path- ${DB2_HANG_DET_SCRIPT_PATH}"
                err_flag=1
            fi
        fi

        #Verifies if DB exists or not only if DB is on local. 
        verify_db_path_on_shared_storage
        if [[ $? -eq 1 ]]; then
             err_flag=1
        fi

		
        #Verify defined value for the DB2_HANG_DETECT_EMAIL attribute
        if [[ -z ${SG_EMAIL_ID} ]]; then
           sg_log 0  "ERROR: Please specify value for email_id attribute"
           err_flag=1
        fi

     fi
  fi


  # This is to ensure that DB2 time out value specified should be lesser than the package timeout value.
  if [[ ${DB2_TIME_OUT} -gt ${SG_HALT_SCRIPT_TIMEOUT} && ${SG_HALT_SCRIPT_TIMEOUT} != "no_timeout" ]]; then
     sg_log 0 "ERROR: DB2 timeout value ${DB2_TIME_OUT} should be lesser than"\
     "SG_HALT_SCRIPT_TIMEOUT value ${SG_HALT_SCRIPT_TIMEOUT}" 
     err_flag=1
  fi

  if [[ $err_flag -eq 1 ]]; then
     return 1
  fi

  sg_log 0 "DB2 Module verification succeeded."
  return 0

}
###############################################################################
# Function: db2_run_cmds
# Description: Starts the db2 instance defined in the package configuration file
# Arguments  : None
# Returns    : 0 - on successful start
#              1 - on failure during startup 
###############################################################################


function db2_run_cmds
{
  # DB2 Startup
  typeset -i exit_status=0
    
  # Execute "db2gcf -u" on every partition specified to bring up the partition
  ${SU} - ${DB2_INSTANCE_USER} -c "/bin/sh" << DB2_USER_LOGIN    
  ret_val=0
  for i in ${DB2_PARTITION_NUMBER[@]}
  do
    echo "$(date '+%b %e %T') - Node \"${HOST}\" Starting DB2 instance $DB2_INSTANCE_NAME Partition \$i"
    $INSTANCE_HOME/sqllib/bin/db2gcf -u -t 300 -i $DB2_INSTANCE_NAME -p \${i}
    ret_val=\$?
    case \$ret_val in
     0)
        echo "$(date '+%b %e %T') - Node \"${HOST}\" INFO: DB2 startup done successfully for partition \${i}." 
        ;;
     1) echo "$(date '+%b %e %T') - Node \"${HOST}\" ERROR: DB2 status is not avaiable in the partition \${i}".
        break 
        ;;
     2) echo "$(date '+%b %e %T') - Node \"${HOST}\" ERROR: db2gcf has been called with wrong number of parameters for partition \${i}."
        break 
        ;;
     3) echo "$(date '+%b %e %T') - Node \"${HOST}\" ERROR: db2gcf gcfmodule failed to execute the requested service for partition \${i}."
        break
        ;; 
     esac
  done
    exit \$ret_val
DB2_USER_LOGIN
   
   exit_status=$?
   case $exit_status in
     0)
        log_message "INFO: DB2 startup completed successfully."
        return 0;
        ;;
     *)
        log_message "ERROR: DB2 startup failed."
        check_return 1 1
        ;;        
    esac
}




#############################################################################
# Function: db2_halt_cmds
# Description: Shutdown function of DB2 uses db2gcf method to halt any 
#              process that will not die normally after waiting for the 
#              TIME_OUT period can be killed.
#############################################################################
function db2_halt_cmds
{
 #   Call this script with the kill option to terminate any process that will
 #   not die normally after waiting for the TIME_OUT period.
 #   By calling kill and sending the process to background makes the invoking
 #   shell proceed with the abort operation. If the abort operation does not
 #   complete within TIME_OUT period, the background process terminates all
 #   lingering processes.
    set -m
    ${0} kill &


 # If the server is up and running, then we shut it down.
    
 typeset -i exit_status=0
    
 # Execute "db2gcf -d" on every partition specified to stop the partition
    ${SU} - ${DB2_INSTANCE_USER} -c "/bin/sh" << DB2_USER_LOGIN
    for i in ${DB2_PARTITION_NUMBER[@]}
    do
      echo "$(date '+%b %e %T') - Node \"${HOST}\" Stopping DB2 Instance $DB2_INSTANCE_NAME Partition \${i}"
      $INSTANCE_HOME/sqllib/bin/db2gcf -d -t 300 -i $DB2_INSTANCE_NAME -p \${i}
      ret_val=\$?
      if [[ \$ret_val -ne 0 ]]
      then
        break 
      fi
    done
    exit \$ret_val
DB2_USER_LOGIN
    exit_status=$?

   if [[ ${exit_status} -ne 0 ]]
   then
      log_message "ERROR: Failed to halt DB2 server ${DB2_INSTANCE_NAME} gracefully."
   else
      log_message "Successfully Halted DB2 server ${DB2_INSTANCE_NAME}."
   fi



   #   Make sure all processes have gone away before determining that shutdown is
   #   complete.  This stops the other node from starting up the package before it
   #   has been stopped and the file system has been unmounted.

    typeset -i c
    typeset -i num_procs=${#DB2_MONITOR_PROCESSES[@]}

    while true
    do
        for i in ${DB2_MONITOR_PROCESSES[@]}
        do
          flag=0
          
          for j in ${DB2_PARTITION_NUMBER[@]}
          do
            id=`ps -fu "${DB2_INSTANCE_NAME}" | grep ${i} | grep "${j}[ ]*$" | awk '/'${i}'/ { print $2 }'`
            if [[ ${id} != "" ]]
            then
               flag=1           
            fi
          done
            
          if [[ ${flag} -eq 0 ]]
             then
                log_message "*** ${i} process is not running. ***"
                c=0
                while (( c < $num_procs ))
                do
                    if [[ ${DB2_MONITOR_PROCESSES[$c]} = $i ]]
                    then
                        unset DB2_MONITOR_PROCESSES[$c]
                        c=$num_procs
                    fi
                    (( c = c + 1 ))
                done            
          fi          
        done

        if [[ ${DB2_MONITOR_PROCESSES[@]} = "" ]]
        then
            job=$(jobs | grep -v "Done")
            if [[ ${job} != "" ]]
            then
                log_message "Killing the kill_hung_processes background process"
                kill %1
            fi
            return 0
        fi
        sleep 5
     done
}

###############################################################################
# Function    : reconfigure_start_function
# Description : This function stops the DB2 package service during online
#               reconfiguration.
# Arguments   : None
# Returns     : None on success. 
#               1 - on failure
###############################################################################
function reconfigure_start_function
{
    validate_reconfigure_restart
    if [[ $? -ne 0 ]]
    then
        get_service_name
        msg="cmrunserv: Unable to start service $service_name (it is already active)"
        out=`${SGSBIN}/cmrunserv ${service_name} "${service_cmd}" 2>&1`
        exit_code=$?
        if [[ $exit_code -ne 0 ]] && [[ "$msg" != "$out" ]] ;
        then
            sg_log 999 "ERROR: Failed to start the service $service_name for online package modification."
            sg_set_reconfig_failure
            return 1
        elif [[ $exit_code -eq 0 ]] ; then
            sg_log 999 "SUCCESS: Successfully started the service $service_name for online package modification."
            if ! sg_check_if_reconfig_has_failed ; then
                 sg_log 0 "SUCCESS: Succesfully started service $service_name for online package modification. "
            fi
        fi
    fi
    return $exit_code
}



###############################################################################
# Function    : reconfigure_stop_function
# Description : This function starts the DB2 package service during online
#               reconfiguration.
# Arguments   : None
# Returns     : None on success.
#               1 - on failure
###############################################################################
function reconfigure_stop_function
{
    exit_code=0
    if sg_check_if_reconfig_has_failed
    then
        sg_log 999 "WARNING: Not attempting to stop the IBM DB2 Monitoring service $service_name for online package modification."
        return 0
    fi

    validate_reconfigure_restart
    if [[ $? -ne 0 ]]
    then
        # Find the monitoring Service name
        get_service_name

        if ! ${SGSBIN}/cmhaltserv ${service_name} ; then
           sg_log 999 "ERROR: Could not stop  the service $service_name for online package modification."
           sg_set_reconfig_failure
           exit_code=1
        else
           sg_log 999 "SUCCESS: Successfully stopped the service $service_name online package modification."
           if ! sg_check_if_reconfig_has_failed ; then
                sg_log 0 "SUCCESS: Successfully stopped the service $service_name for online package modification."
           fi
        fi
    fi
    return $exit_code
}


###############################################################################
# MAIN
# Check the command-line option and take the appropriate action.
###############################################################################
export PATH=$PATH:/bin:/sbin:/usr/bin:/usr/sbin:/usr/lbin
SU=su
exit_code=0

# Confirm that INSTANCE_NAME variable contains non-null string 
if [[ $1 != "validate" ]]; then
  log_message "*** $0 called with $1 argument. ***"
fi
case $1 in

    kill)
        kill_hung_processes
        exit_code=$?
    ;;

    db2_monitor)
        .$SGROOT/db2toolkit/hadb2.mon
    ;;

    start)
        log_message ": Starting DB2 Instance $DB2_INSTANCE_NAME by user ${DB2_INSTANCE_USER}"
        db2_run_cmds
        exit_code=$?
    ;;

    stop)
        log_message ": Halting DB2 Instance $DB2_INSTANCE_NAME by user ${DB2_INSTANCE_USER}"
        db2_halt_cmds
        exit_code=$?
    ;;
   
    validate) 
        validate_db2
        exit_code=$?
    ;;
  
    reconfigure_start)
        reconfigure_start_function
        exit_code=$?
    
    ;;
    reconfigure_stop)
       reconfigure_stop_function
       exit_code=$?
    ;;
    *)
        log_message "Usage: ${0} [ start | stop | monitor ]"
    ;;
esac
exit $exit_code 
